import subprocess
import re
import importlib.metadata
import os
import sys
from typing import List, Dict, Tuple, Optional

# ANSI 颜色代码
class Colors:
    RED = '\033[91m'
    GREEN = '\033[92m'
    YELLOW = '\033[93m'
    BLUE = '\033[94m'
    MAGENTA = '\033[95m'
    CYAN = '\033[96m'
    RESET = '\033[0m'

def find_requirements_file():
    """从当前目录向上查找 requirements.txt"""
    current_dir = os.getcwd()
    while True:
        file_path = os.path.join(current_dir, 'requirements.txt')
        if os.path.isfile(file_path):
            return file_path
        parent_dir = os.path.dirname(current_dir)
        if parent_dir == current_dir:
            print(f"{Colors.RED}错误：未找到 requirements.txt 文件。{Colors.RESET}")
            return None
        current_dir = parent_dir

def read_requirements(file_path: str) -> List[str]:
    """读取 requirements.txt，自动处理编码问题（含 chardet 自动安装）"""
    try:
        with open(file_path, 'rb') as f:
            content = f.read()
            try:
                return [line.strip() for line in content.decode('utf-8').splitlines()
                        if line.strip() and not line.startswith('#')]
            except UnicodeDecodeError:
                # 自动安装 chardet 模块
                try:
                    import chardet
                except ModuleNotFoundError:
                    print(f"{Colors.YELLOW}检测到缺少 chardet，正在自动安装...{Colors.RESET}")
                    process = subprocess.run(
                        [sys.executable, "-m", "pip", "install", "chardet"],
                        check=False,
                        text=True
                    )
                    if process.returncode != 0:
                        print(f"{Colors.RED}错误：无法安装 chardet，请手动安装后重试。{Colors.RESET}")
                        return []
                    import chardet  # 重新导入已安装的 chardet
                
                encoding = chardet.detect(content)['encoding'] or 'utf-8'
                return [line.strip() for line in content.decode(encoding).splitlines()
                        if line.strip() and not line.startswith('#')]
    except FileNotFoundError:
        print(f"{Colors.RED}错误：未找到 {file_path} 文件。{Colors.RESET}")
        return []

def install_module(requirement: str) -> bool:
    print(f"{Colors.CYAN}正在安装: {requirement}{Colors.RESET}")
    try:
        process = subprocess.run(
            ["pip", "install", requirement],
            check=False,
            text=True
        )
        return process.returncode == 0
    except Exception as e:
        print(f"{Colors.RED}安装出错: {str(e)}{Colors.RESET}")
        return False

def get_installed_version(package: str) -> Optional[str]:
    try:
        return importlib.metadata.version(package)
    except importlib.metadata.PackageNotFoundError:
        return None

def parse_requirement(requirement: str) -> Tuple[Optional[str], Optional[str]]:
    match = re.match(
        r'^([a-zA-Z0-9][a-zA-Z0-9\-_.]+)\s*(?:==|>=|<=|!=|<|>|~=)?\s*([^;\s]*)', 
        requirement
    )
    if not match:
        return None, None
    return match.group(1), match.group(2) if match.group(2) else None

def main():
    requirements_file = find_requirements_file()
    if not requirements_file:
        # 如果找不到文件，尝试在当前目录创建
        print(f"{Colors.YELLOW}在当前目录下创建 requirements.txt{Colors.RESET}")
        with open('requirements.txt', 'w') as f:
            f.write("asgiref==3.8.1\nattrs==25.3.0\ncertifi==2025.4.26\ncffi==1.17.1\nDjango==5.2.1\nexceptiongroup==1.3.0\nh11==0.16.0\nidna==3.10\noutcome==1.3.0.post0\npycparser==2.22\nPyMySQL==1.1.1\nPySocks==1.7.1\nselenium==4.32.0\nsniffio==1.3.1\nsortedcontainers==2.4.0\nsqlparse==0.5.3\ntrio==0.30.0\ntrio-websocket==0.12.2\ntyping_extensions==4.13.2\ntzdata==2025.2\nurllib3==2.4.0\nwebsocket-client==1.8.0\nwsproto==1.2.0")
        requirements_file = 'requirements.txt'

    requirements = read_requirements(requirements_file)
    
    # 显示读取到的依赖项
    print(f"{Colors.YELLOW}读取到以下依赖项:{Colors.RESET}")
    for req in requirements:
        print(f"  {req}")
    
    installed_count = 0
    already_installed = {}  # 包名: 版本（需求版本）
    replaced = {}  # 包名: (需求版本, 实际安装版本)
    skipped = []  # 原始requirement字符串
    missing = []  # 未成功安装的包

    print(f"{Colors.YELLOW}\n=== 开始安装依赖包 ==={Colors.RESET}")

    # 首先确保安装 PyMySQL
    print(f"{Colors.YELLOW}优先安装 PyMySQL...{Colors.RESET}")
    pymysql_success = install_module("PyMySQL==1.1.1")
    if pymysql_success:
        installed_count += 1
        print(f"{Colors.GREEN}✓ PyMySQL 安装成功{Colors.RESET}")
    else:
        print(f"{Colors.RED}× PyMySQL 安装失败，尝试安装其他依赖{Colors.RESET}")
        missing.append("PyMySQL==1.1.1")

    for requirement in requirements:
        package, req_version = parse_requirement(requirement)
        if not package:
            skipped.append(requirement)
            continue

        # 如果是 PyMySQL，已经处理过了，跳过
        if package.lower() == 'pymysql':
            continue

        # 检查是否已安装（按需求版本判断）
        installed_version = get_installed_version(package)
        if installed_version:
            if req_version and installed_version == req_version:
                already_installed[package] = req_version
                print(f"{Colors.GREEN}√ 已存在: {package}=={req_version}{Colors.RESET}")
                continue  # 只有在正确版本已安装的情况下才继续
            else:
                print(f"{Colors.YELLOW}需要更新: {package} (当前:{installed_version}, 需要:{req_version}){Colors.RESET}")
                # 如果版本不一致，继续安装正确的版本

        # 尝试安装（带需求版本）
        install_success = install_module(requirement)
        actual_version = get_installed_version(package) if install_success else None
        
        if install_success:
            installed_count += 1
            # 记录版本差异
            if req_version and actual_version and req_version != actual_version:
                replaced[package] = (req_version, actual_version)
            elif not req_version and actual_version:  # 未指定版本时记录实际安装版本
                replaced[package] = ('未指定', actual_version)
            continue

        # 尝试安装最新版（不带版本号）
        print(f"{Colors.YELLOW}尝试安装 {package} 最新版本...{Colors.RESET}")
        install_latest_success = install_module(package)
        actual_latest_version = get_installed_version(package) if install_latest_success else None
        
        if install_latest_success:
            installed_count += 1
            replaced[package] = (req_version or '未指定', actual_latest_version)
            continue

        missing.append(requirement)
        print(f"{Colors.RED}× 安装失败: {requirement}{Colors.RESET}")

    # 输出总结
    print(f"\n{Colors.YELLOW}=== 安装结果总结 ==={Colors.RESET}")
    print(f"{Colors.GREEN}成功安装: {installed_count} 个包{Colors.RESET}")

    if already_installed:
        print(f"{Colors.BLUE}已存在无需安装: {len(already_installed)} 个包{Colors.RESET}")

    if replaced:
        print(f"\n{Colors.MAGENTA}版本替换情况:{Colors.RESET}")
        for pkg, (old_ver, new_ver) in replaced.items():
            old_ver_display = old_ver if old_ver else '未指定'
            print(f"  {pkg}: {Colors.YELLOW}{old_ver_display}{Colors.RESET} → {Colors.GREEN}{new_ver}{Colors.RESET}")

    if missing:
        print(f"\n{Colors.RED}安装失败: {len(missing)} 个包{Colors.RESET}")
        for req in missing:
            print(f"  {req}")

    if skipped:
        print(f"\n{Colors.RED}解析失败: {len(skipped)} 个包{Colors.RESET}")
        for req in skipped:
            print(f"  {req}")

    # 最后再次检查关键包 PyMySQL 是否已安装
    pymysql_version = get_installed_version("pymysql")
    if pymysql_version:
        print(f"\n{Colors.GREEN}确认 PyMySQL {pymysql_version} 已成功安装{Colors.RESET}")
    else:
        print(f"\n{Colors.RED}警告: PyMySQL 未安装！尝试最后一次安装...{Colors.RESET}")
        install_module("PyMySQL")
        if get_installed_version("pymysql"):
            print(f"{Colors.GREEN}PyMySQL 安装成功！{Colors.RESET}")
        else:
            print(f"{Colors.RED}PyMySQL 安装失败，请手动安装后再继续！{Colors.RESET}")

    print(f"\n{Colors.YELLOW}=== 安装完成 ==={Colors.RESET}")

if __name__ == "__main__":
    main()